# Tweepy
# Copyright 2009-2023 Joshua Roesslein
# See LICENSE for details.

"""
Tweepy Twitter API library
"""
__version__ = '4.14.0'
__author__ = 'Joshua Roesslein'
__license__ = 'MIT'

from tweepy.api import API
from tweepy.auth import (
    AppAuthHandler, OAuthHandler, OAuth1UserHandler, OAuth2AppHandler,
    OAuth2BearerHandler, OAuth2UserHandler
)
from tweepy.cache import Cache, FileCache, MemoryCache
from tweepy.client import Client, Response
from tweepy.cursor import Cursor
from tweepy.direct_message_event import (
    DirectMessageEvent, DIRECT_MESSAGE_EVENT_FIELDS, DM_EVENT_FIELDS
)
from tweepy.errors import (
    BadRequest, Forbidden, HTTPException, NotFound, TooManyRequests,
    TweepyException, TwitterServerError, Unauthorized
)
from tweepy.list import List, LIST_FIELDS
from tweepy.media import Media, MEDIA_FIELDS
from tweepy.pagination import Paginator
from tweepy.place import Place, PLACE_FIELDS
from tweepy.poll import Poll, POLL_FIELDS
from tweepy.space import PUBLIC_SPACE_FIELDS, Space, SPACE_FIELDS
from tweepy.streaming import (
    StreamingClient, StreamResponse, StreamRule
)
from tweepy.tweet import (
    PUBLIC_TWEET_FIELDS, ReferencedTweet, Tweet, TWEET_FIELDS
)
from tweepy.user import User, USER_FIELDS

# Global, unauthenticated instance of API
api = API()



# Tweepy
# Copyright 2009-2023 Joshua Roesslein
# See LICENSE for details.

from math import inf

import aiohttp

from tweepy.client import Response


class AsyncPaginator:
    """AsyncPaginator( \
        self, method, *args, limit=inf, pagination_token=None, **kwargs \
    )

    :class:`AsyncPaginator` can be used to paginate for any
    :class:`AsyncClient` methods that support pagination

    .. note::

        When the returned response from the method being passed is of type
        :class:`aiohttp.ClientResponse`, it will be deserialized in order to
        parse the pagination tokens, likely negating any potential performance
        benefits from using a :class:`aiohttp.ClientResponse` return type.

    .. versionadded:: 4.11

    Parameters
    ----------
    method
        :class:`AsyncClient` method to paginate for
    args
        Positional arguments to pass to ``method``
    limit
        Maximum number of requests to make to the API
    pagination_token
        Pagination token to start pagination with
    kwargs
        Keyword arguments to pass to ``method``
    """

    def __init__(self, method, *args, **kwargs):
        self.method = method
        self.args = args
        self.kwargs = kwargs

    def __aiter__(self):
        return AsyncPaginationIterator(self.method, *self.args, **self.kwargs)

    def __reversed__(self):
        return AsyncPaginationIterator(
            self.method, *self.args, reverse=True, **self.kwargs
        )

    async def flatten(self, limit=inf):
        """Flatten paginated data

        Parameters
        ----------
        limit
            Maximum number of results to yield
        """
        if limit <= 0:
            return

        count = 0
        async for response in AsyncPaginationIterator(
            self.method, *self.args, **self.kwargs
        ):
            if isinstance(response, Response):
                response_data = response.data or []
            elif isinstance(response, dict):
                response_data = response.get("data", [])
            else:
                raise RuntimeError(
                    "AsyncPaginator.flatten does not support the "
                    f"{type(response)} return type for "
                    f"{self.method.__qualname__}"
                )
            for data in response_data:
                yield data
                count += 1
                if count == limit:
                    return


class AsyncPaginationIterator:

    def __init__(
        self, method, *args, limit=inf, pagination_token=None, reverse=False,
        **kwargs
    ):
        self.method = method
        self.args = args
        self.limit = limit
        self.kwargs = kwargs
        self.reverse = reverse

        if reverse:
            self.previous_token = pagination_token
            self.next_token = None
        else:
            self.previous_token = None
            self.next_token = pagination_token

        self.count = 0

    def __aiter__(self):
        return self

    async def __anext__(self):
        if self.reverse:
            pagination_token = self.previous_token
        else:
            pagination_token = self.next_token

        if self.count >= self.limit or self.count and pagination_token is None:
            raise StopAsyncIteration

        # https://twittercommunity.com/t/why-does-timeline-use-pagination-token-while-search-uses-next-token/150963
        if self.method.__name__ in (
            "search_all_tweets", "search_recent_tweets",
            "get_all_tweets_count"
        ):
            self.kwargs["next_token"] = pagination_token
        else:
            self.kwargs["pagination_token"] = pagination_token

        response = await self.method(*self.args, **self.kwargs)

        if isinstance(response, Response):
            meta = response.meta
        elif isinstance(response, dict):
            meta = response.get("meta", {})
        elif isinstance(response, aiohttp.ClientResponse):
            meta = (await response.json()).get("meta", {})
        else:
            raise RuntimeError(
                f"Unknown {type(response)} return type for "
                f"{self.method.__qualname__}"
            )

        self.previous_token = meta.get("previous_token")
        self.next_token = meta.get("next_token")
        self.count += 1

        return response



# Tweepy
# Copyright 2009-2023 Joshua Roesslein
# See LICENSE for details.

from tweepy.mixins import DataMapping, HashableID
from tweepy.utils import parse_datetime


#: All the potential fields for :class:`List` objects
LIST_FIELDS = [
    "created_at",
    "description",
    "follower_count",
    "id",
    "member_count",
    "name",
    "owner_id",
    "private",
]


class List(HashableID, DataMapping):
    """The list object contains `Twitter Lists`_ metadata describing the
    referenced List. The List object is the primary object returned in the List
    lookup endpoint. When requesting additional List fields on this endpoint,
    simply use the fields parameter ``list.fields``.

    At the moment, the List object cannot be found as a child object from any
    other data object. However, user objects can be found and expanded in the
    user resource. These objects are available for expansion by adding
    ``owner_id`` to the ``expansions`` query parameter. Use the expansion with
    the field parameter: ``list.fields`` when requesting additional fields to
    complete the primary List object and ``user.fields`` to complete the
    expansion object.

    .. versionadded:: 4.4

    Attributes
    ----------
    data : dict
        The JSON data representing the List.
    id : str
        The unique identifier of this List.
    name : str
        The name of the List, as defined when creating the List.
    created_at : datetime.datetime | None
        The UTC datetime that the List was created on Twitter.
    description : str | None
        A brief description to let users know about the List.
    follower_count : int | None
        Shows how many users follow this List,
    member_count : int | None
        Shows how many members are part of this List.
    private : bool | None
        Indicates if the List is private.
    owner_id : str | None
        Unique identifier of this List's owner.

    References
    ----------
    https://developer.twitter.com/en/docs/twitter-api/data-dictionary/object-model/lists

    .. _Twitter Lists: https://help.twitter.com/en/using-twitter/twitter-lists
    """

    __slots__ = (
        "data", "id", "name", "created_at", "description", "follower_count",
        "member_count", "private", "owner_id"
    )

    def __init__(self, data):
        self.data = data
        self.id = data["id"]
        self.name = data["name"]

        self.created_at = data.get("created_at")
        if self.created_at is not None:
            self.created_at = parse_datetime(self.created_at)

        self.description = data.get("description")
        self.follower_count = data.get("follower_count")
        self.member_count = data.get("member_count")
        self.private = data.get("private")
        self.owner_id = data.get("owner_id")

    def __repr__(self):
        return f"<List id={self.id} name={self.name}>"

    def __str__(self):
        return self.name



import tweepy


bearer_token = ""

client = tweepy.Client(bearer_token)

# Get Tweets

# This endpoint/method returns a variety of information about the Tweet(s)
# specified by the requested ID or list of IDs

tweet_ids = [1460323737035677698, 1293593516040269825, 1293595870563381249]

# By default, only the ID and text fields of each Tweet will be returned
# Additional fields can be retrieved using the tweet_fields parameter
response = client.get_tweets(tweet_ids, tweet_fields=["created_at"])

for tweet in response.data:
    print(tweet.id, tweet.created_at)



# Tweepy
# Copyright 2009-2023 Joshua Roesslein
# See LICENSE for details.

from tweepy.mixins import DataMapping, HashableID
from tweepy.tweet import ReferencedTweet
from tweepy.utils import parse_datetime


#: All the potential fields for :class:`DirectMessageEvent` objects
DIRECT_MESSAGE_EVENT_FIELDS = [
    "attachments",
    "created_at",
    "dm_conversation_id",
    "event_type",
    "id",
    "participant_ids",
    "referenced_tweets",
    "sender_id",
    "text",
]
#: Alias for :const:`DIRECT_MESSAGE_EVENT_FIELDS`
DM_EVENT_FIELDS = DIRECT_MESSAGE_EVENT_FIELDS


class DirectMessageEvent(HashableID, DataMapping):
    """Direct Message (DM) conversations are made up of events. The Twitter API
    v2 currently supports three event types: MessageCreate, ParticipantsJoin,
    and ParticipantsLeave.

    DM event objects are returned by the `Direct Message lookup`_ endpoints,
    and a MessageCreate event is created when Direct Messages are successfully
    created with the `Manage Direct Messages`_ endpoints.

    When requesting DM events, there are three default event object attributes,
    or fields, included: ``id``, ``event_type``, and ``text``. To receive
    additional event `fields`_, use the fields parameter ``dm_event.fields`` to
    select others. Other available event fields include the following:
    ``dm_conversation_id``, ``created_at``, ``sender_id`, ``attachments``,
    ``participant_ids``, and ``referenced_tweets``.

    Several of these fields provide the IDs of other Twitter objects related to
    the Direct Message event:

    * ``sender_id`` - The ID of the account that sent the message, or who
      invited a participant to a group conversation
    * ``partricipants_ids`` - An array of account IDs. For ParticipantsJoin and
      ParticipantsLeave events this array will contain a single ID of the
      account that created the event
    * ``attachments`` - Provides media IDs for content that has been uploaded
      to Twitter by the sender
    * ``referenced_tweets`` - If a Tweet URL is found in the text field, the ID
      of that Tweet is included in the response

    The ``sender_id``, ``participant_ids``, ``referenced_tweets.id``, and
    ``attachments.media_keys`` `expansions`_ are available to expand on these
    Twitter object IDs.

    .. versionadded:: 4.12

    Attributes
    ----------
    data : dict
        The JSON data representing the Direct Message event.
    id : int
        The unique identifier of the event.
    event_type : str
        Describes the type of event. Three types are currently supported:

        * MessageCreate
        * ParticipantsJoin
        * ParticipantsLeave
    text : str | None
        The actual UTF-8 text of the Direct Message. 
    sender_id : int | None
        ID of the User creating the event. To expand this object in the
        response, include ``sender_id`` as an expansion and use the
        ``user.fields`` query parameter to specify User object attributes of
        interest.
    participant_ids : list[int] | None
        IDs of the participants joining and leaving a group conversation. Also
        used when creating new group conversations. To expand this object in
        the response, include ``participant_ids`` as an expansion and use the
        ``user.fields`` query parameter to specify User object attributes of
        interest.
    dm_conversation_id : str | None
        The unique identifier of the conversation the event is apart of.
    created_at : datetime.datetime | None
        Creation time (UTC) of the Tweet.
    referenced_tweets : list[ReferencedTweet] | None
        ID for any Tweet mentioned in the Direct Message text. To expand this
        object in the response, include ``referenced_tweets.id`` as an
        expansion and use the ``tweet.fields`` query parameter to specify Tweet
        object attributes of interest.
    attachments : dict | None
        For Direct Messages with attached Media, provides the media key of the
        uploaded content (photo, video, or GIF. To expand this object in the
        response, include ``attachments.media_keys`` as an expansion and use
        the ``media.fields`` query parameter to specify media object attributes
        of interest. Currently, one attachment is supported. 

    References
    ----------
    https://developer.twitter.com/en/docs/twitter-api/data-dictionary/object-model/dm-events

    .. _Direct Message lookup: https://developer.twitter.com/en/docs/twitter-api/direct-messages/lookup/introduction
    .. _Manage Direct Messages: https://developer.twitter.com/en/docs/twitter-api/direct-messages/manage/introduction
    .. _fields: https://developer.twitter.com/en/docs/twitter-api/fields
    .. _expansions: https://developer.twitter.com/en/docs/twitter-api/expansions
    """

    __slots__ = (
        "data", "id", "event_type", "attachments", "created_at",
        "dm_conversation_id", "participant_ids", "referenced_tweets",
        "sender_id", "text"
    )

    def __init__(self, data):
        self.data = data
        self.id = int(data["id"])
        self.event_type = data["event_type"]

        self.text = data.get("text")

        self.sender_id = data.get("sender_id")
        if self.sender_id is not None:
            self.sender_id = int(self.sender_id)

        self.participant_ids = data.get("participant_ids")
        if self.participant_ids is not None:
            self.participant_ids = list(map(int, self.participant_ids))

        self.dm_conversation_id = data.get("dm_conversation_id")

        self.created_at = data.get("created_at")
        if self.created_at is not None:
            self.created_at = parse_datetime(self.created_at)

        self.referenced_tweets = data.get("referenced_tweets")
        if self.referenced_tweets is not None:
            self.referenced_tweets = [
                ReferencedTweet(referenced_tweet)
                for referenced_tweet in self.referenced_tweets
            ]

        self.attachments = data.get("attachments")

    def __repr__(self):
        representation = (
            f"<Direct Message Event id={self.id} event_type={self.event_type}"
        )
        if self.text is not None:
            representation += f" text={repr(self.text)}"
        representation += '>'
        return representation

    def __str__(self):
        return self.text or self.__repr__()
